@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-gst.c,v 1.41 2003/06/27 20:39:28 dolphy Exp $
+ * $Id: bacon-video-widget-gst.c,v 1.51 2003/10/07 20:33:04 hadess Exp $
  *
  */
 
@@ -65,6 +65,7 @@ enum
   TICK,
   GOT_METADATA,
   BUFFERING,
+  SPEED_WARNING,
   LAST_SIGNAL
 };
 
@@ -79,7 +80,6 @@ enum
 {
   PROP_0,
   PROP_LOGO_MODE,
-  PROP_SPEED,
   PROP_POSITION,
   PROP_CURRENT_TIME,
   PROP_STREAM_LENGTH,
@@ -101,10 +101,10 @@ struct BaconVideoWidgetPrivate
 
   gboolean media_has_video;
 
-  guint stream_length;
+  gint64 stream_length;
   gint64 current_time_nanos;
-  guint current_time;
-  guint current_position;
+  gint64 current_time;
+  float current_position;
 
   GHashTable *metadata_hash;
 
@@ -149,9 +149,6 @@ static void bacon_video_widget_get_property (GObject * object,
 
 static void bacon_video_widget_finalize (GObject * object);
 
-static gboolean bacon_video_widget_expose (GtkWidget * widget,
-					   GdkEventExpose * event);
-
 static GtkWidgetClass *parent_class = NULL;
 
 static int bvw_table_signals[LAST_SIGNAL] = { 0 };
@@ -214,7 +211,7 @@ bacon_video_widget_class_init (BaconVideoWidgetClass * klass)
   object_class = (GObjectClass *) klass;
   widget_class = (GtkWidgetClass *) klass;
 
-  parent_class = gtk_type_class (gtk_vbox_get_type ());
+  parent_class = gtk_type_class (gtk_box_get_type ());
 
   /* GtkWidget */
   widget_class->size_request = bacon_video_widget_size_request;
@@ -230,18 +227,13 @@ bacon_video_widget_class_init (BaconVideoWidgetClass * klass)
 				   g_param_spec_boolean ("logo_mode", NULL,
 							 NULL, FALSE,
 							 G_PARAM_READWRITE));
-  g_object_class_install_property (object_class, PROP_SPEED,
-				   g_param_spec_int ("speed", NULL, NULL,
-						     SPEED_PAUSE,
-						     SPEED_NORMAL, 0,
-						     G_PARAM_READWRITE));
   g_object_class_install_property (object_class, PROP_POSITION,
 				   g_param_spec_int ("position", NULL, NULL,
-						     0, 65535, 0,
+						     0, G_MAXINT, 0,
 						     G_PARAM_READABLE));
   g_object_class_install_property (object_class, PROP_STREAM_LENGTH,
-				   g_param_spec_int ("stream_length", NULL,
-						     NULL, 0, 65535, 0,
+				   g_param_spec_int64 ("stream_length", NULL,
+						     NULL, 0, G_MAXINT64, 0,
 						     G_PARAM_READABLE));
   g_object_class_install_property (object_class, PROP_PLAYING,
 				   g_param_spec_boolean ("playing", NULL,
@@ -310,8 +302,8 @@ bacon_video_widget_class_init (BaconVideoWidgetClass * klass)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (BaconVideoWidgetClass, tick),
 		  NULL, NULL,
-		  baconvideowidget_marshal_VOID__INT_INT_INT,
-		  G_TYPE_NONE, 3, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT);
+		  baconvideowidget_marshal_VOID__INT64_INT64_FLOAT,
+		  G_TYPE_NONE, 3, G_TYPE_INT64, G_TYPE_INT64, G_TYPE_FLOAT);
 
   bvw_table_signals[BUFFERING] =
     g_signal_new ("buffering",
@@ -320,13 +312,22 @@ bacon_video_widget_class_init (BaconVideoWidgetClass * klass)
 		  G_STRUCT_OFFSET (BaconVideoWidgetClass, buffering),
 		  NULL, NULL,
 		  g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
+
+  bvw_table_signals[SPEED_WARNING] =
+    g_signal_new ("speed-warning",
+		  G_TYPE_FROM_CLASS (object_class),
+		  G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (BaconVideoWidgetClass, speed_warning),
+		  NULL, NULL,
+		  g_cclosure_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
 }
 
 static void
 bacon_video_widget_instance_init (BaconVideoWidget * bvw)
 {
   int argc = 1;
-  char *argv[] = { "bacon_name", NULL };
+  char **argv = NULL;
 
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
@@ -410,6 +411,23 @@ got_video_size (GstPlay * play, gint width, gint height,
     gst_video_widget_set_source_size (bvw->priv->vw, width, height);
 }
 
+static void
+got_vis_video_size (GstPlay * play, gint width, gint height,
+		    BaconVideoWidget * bvw)
+{
+  g_return_if_fail (bvw != NULL);
+  g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+
+  if (bvw->priv->media_has_video)
+    return;
+  
+  bvw->priv->video_width = width;
+  bvw->priv->video_height = height;
+
+  if (bvw->priv->vw)
+    gst_video_widget_set_source_size (bvw->priv->vw, width, height);
+}
+
 static void
 got_eos (GstPlay * play, BaconVideoWidget * bvw)
 {
@@ -425,8 +443,8 @@ got_stream_length (GstPlay * play, gint64 length_nanos,
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 
-  bvw->priv->stream_length = (length_nanos / GST_MSECOND);
-
+  bvw->priv->stream_length = (gint64) length_nanos / GST_MSECOND;
+  
   g_signal_emit (G_OBJECT (bvw), bvw_table_signals[GOT_METADATA], 0, NULL);
 }
 
@@ -436,17 +454,19 @@ got_time_tick (GstPlay * play, gint64 time_nanos, BaconVideoWidget * bvw)
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 
+  if (bvw->priv->logo_mode != FALSE)
+    return;
+
   bvw->priv->current_time_nanos = time_nanos;
 
-  bvw->priv->current_time = (time_nanos / GST_MSECOND);
+  bvw->priv->current_time = (gint64) time_nanos / GST_MSECOND;
 
   if (bvw->priv->stream_length == 0)
     bvw->priv->current_position = 0;
   else
     {
       bvw->priv->current_position =
-	(long long) bvw->priv->current_time * 65535
-	/ bvw->priv->stream_length;
+	(float) bvw->priv->current_time / bvw->priv->stream_length;
     }
 
   g_signal_emit (G_OBJECT (bvw),
@@ -466,7 +486,7 @@ got_error (GstPlay * play, GstElement * orig,
   if (bvw->priv->last_error_message)
     {
       /* Let's check the latest error message */
-      if (g_strcasecmp (error_message, bvw->priv->last_error_message) == 0)
+      if (g_ascii_strcasecmp (error_message, bvw->priv->last_error_message) == 0)
 	emit = FALSE;
     }
 
@@ -507,7 +527,7 @@ bacon_video_widget_finalize (GObject * object)
       bvw->priv->vw = NULL;
     }
 
-  if ((bvw->priv->play) && GST_IS_PLAY (bvw->priv->play))
+  if ((bvw->priv->play) && G_IS_OBJECT (bvw->priv->play))
     {
       g_object_unref (bvw->priv->play);
       bvw->priv->play = NULL;
@@ -548,12 +568,6 @@ bacon_video_widget_get_backend_name (BaconVideoWidget * bvw)
   return g_strdup_printf ("GStreamer version %d.%d.%d", major, minor, micro);
 }
 
-gboolean
-bacon_video_widget_eject (BaconVideoWidget * bvw)
-{
-  return FALSE;
-}
-
 int
 bacon_video_widget_get_subtitle (BaconVideoWidget * bvw)
 {
@@ -652,7 +666,6 @@ bacon_video_widget_open (BaconVideoWidget * bvw, const gchar * mrl,
   g_return_val_if_fail (bvw->priv->play != NULL, FALSE);
   g_return_val_if_fail (bvw->priv->mrl == NULL, FALSE);
 
-  g_message ("open %s", mrl);
   bvw->priv->mrl = g_strdup (mrl);
 
   /* Resetting last_error_message to NULL */
@@ -673,8 +686,7 @@ bacon_video_widget_open (BaconVideoWidget * bvw, const gchar * mrl,
       bvw->priv->vis_signal_blocked = FALSE;
     }
 
-  gst_play_connect_visualisation (bvw->priv->play, TRUE);
-
+  gst_video_widget_set_source_size (GST_VIDEO_WIDGET (bvw->priv->vw), 1, 1);
   gst_play_need_new_video_window (bvw->priv->play);
 
   if (g_file_test (mrl, G_FILE_TEST_EXISTS))
@@ -695,15 +707,29 @@ bacon_video_widget_open (BaconVideoWidget * bvw, const gchar * mrl,
   return gst_play_set_location (bvw->priv->play, mrl);;
 }
 
-/* This is used for seeking:
- * @pos is used for seeking, from 0 (start) to 65535 (end)
- * @start_time is in milliseconds */
 gboolean
-bacon_video_widget_play (BaconVideoWidget * bvw,
-			 guint pos, guint start_time, GError ** error)
+bacon_video_widget_play (BaconVideoWidget * bvw, GError ** error)
 {
-  gint64 seek_time;
-  gint64 length_nanos;
+  g_return_val_if_fail (bvw != NULL, FALSE);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
+  g_return_val_if_fail (GST_IS_PLAY (bvw->priv->play), FALSE);
+
+  /* Resetting last_error_message to NULL */
+  if (bvw->priv->last_error_message)
+    {
+      g_free (bvw->priv->last_error_message);
+      bvw->priv->last_error_message = NULL;
+    }
+
+  gst_play_set_state (bvw->priv->play, GST_STATE_PLAYING);
+
+  return TRUE;
+}
+
+gboolean bacon_video_widget_seek (BaconVideoWidget *bvw, float position,
+		GError **gerror)
+{
+  gint64 seek_time, length_nanos;
 
   g_return_val_if_fail (bvw != NULL, FALSE);
   g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
@@ -716,21 +742,29 @@ bacon_video_widget_play (BaconVideoWidget * bvw,
       bvw->priv->last_error_message = NULL;
     }
 
-  if (pos)
+  length_nanos = (gint64) (bvw->priv->stream_length * GST_MSECOND);
+  seek_time = (gint64) (length_nanos * position);
+  gst_play_seek_to_time (bvw->priv->play, seek_time);
+
+  return TRUE;
+}
+
+gboolean bacon_video_widget_seek_time (BaconVideoWidget *bvw, gint64 time,
+		GError **gerror)
+{
+  g_return_val_if_fail (bvw != NULL, FALSE);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
+  g_return_val_if_fail (GST_IS_PLAY (bvw->priv->play), FALSE);
+
+  /* Resetting last_error_message to NULL */
+  if (bvw->priv->last_error_message)
     {
-      length_nanos = (gint64) (bvw->priv->stream_length * GST_MSECOND);
-      seek_time = (gint64) (length_nanos * pos / 65535);
-      gst_play_seek_to_time (bvw->priv->play, seek_time);
-    }
-  else if (start_time)
-    {
-      seek_time = (gint64) (start_time * GST_MSECOND);
-      gst_play_seek_to_time (bvw->priv->play, seek_time);
-    }
-  else
-    {
-      gst_play_set_state (bvw->priv->play, GST_STATE_PLAYING);
+      g_free (bvw->priv->last_error_message);
+      bvw->priv->last_error_message = NULL;
     }
+
+  gst_play_seek_to_time (bvw->priv->play, time);
+
   return TRUE;
 }
 
@@ -797,40 +831,13 @@ bacon_video_widget_get_logo_mode (BaconVideoWidget * bvw)
 }
 
 void
-bacon_video_widget_set_speed (BaconVideoWidget * bvw, Speeds speed)
+bacon_video_widget_pause (BaconVideoWidget * bvw)
 {
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
   g_return_if_fail (GST_IS_PLAY (bvw->priv->play));
 
-  switch (speed)
-    {
-    case SPEED_PAUSE:
-      gst_play_set_state (bvw->priv->play, GST_STATE_PAUSED);
-      break;;
-    case SPEED_NORMAL:
-      gst_play_set_state (bvw->priv->play, GST_STATE_PLAYING);
-      break;;
-    default:
-      g_warning ("unsupported speed %d", speed);
-    }
-}
-
-int
-bacon_video_widget_get_speed (BaconVideoWidget * bvw)
-{
-  g_return_val_if_fail (bvw != NULL, -1);
-  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), -1);
-  g_return_val_if_fail (GST_IS_PLAY (bvw->priv->play), -1);
-
-  if (gst_play_get_state (bvw->priv->play) == GST_STATE_PAUSED)
-    return SPEED_PAUSE;
-  else if (gst_play_get_state (bvw->priv->play) == GST_STATE_PLAYING)
-    return SPEED_NORMAL;
-  else
-    g_message ("pipeline is not in a known Bacon speed");
-
-  return -1;
+  gst_play_set_state (bvw->priv->play, GST_STATE_PAUSED);
 }
 
 void
@@ -872,6 +879,15 @@ void
 bacon_video_widget_set_fullscreen (BaconVideoWidget * bvw,
 				   gboolean fullscreen)
 {
+  g_return_if_fail (bvw != NULL);
+  g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+  
+  if (bvw->priv->vw)
+    {
+      gst_video_widget_set_scale_override (GST_VIDEO_WIDGET
+					   (bvw->priv->vw),
+					   FALSE);
+    }
 }
 
 void
@@ -901,13 +917,13 @@ bacon_video_widget_set_media_device (BaconVideoWidget * bvw, const char *path)
   g_return_if_fail (GST_IS_PLAY (bvw->priv->play));
 }
 
-int
+gboolean
 bacon_video_widget_set_show_visuals (BaconVideoWidget * bvw,
 				     gboolean show_visuals)
 {
-  g_return_if_fail (bvw != NULL);
-  g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
-  g_return_if_fail (GST_IS_PLAY (bvw->priv->play));
+  g_return_val_if_fail (bvw != NULL, FALSE);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
+  g_return_val_if_fail (GST_IS_PLAY (bvw->priv->play), FALSE);
 
   if (!bvw->priv->media_has_video)
     {
@@ -928,9 +944,9 @@ bacon_video_widget_get_visuals_list (BaconVideoWidget * bvw)
   GList *pool_registries = NULL, *registries = NULL, *vis_plugins_list = NULL;
   GList *plugins = NULL, *features = NULL;
 
-  g_return_if_fail (bvw != NULL);
-  g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
-  g_return_if_fail (GST_IS_PLAY (bvw->priv->play));
+  g_return_val_if_fail (bvw != NULL, NULL);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), NULL);
+  g_return_val_if_fail (GST_IS_PLAY (bvw->priv->play), NULL);
 
   /* Cache */
   if (bvw->priv->vis_plugins_list)
@@ -1019,7 +1035,6 @@ bacon_video_widget_set_visuals (BaconVideoWidget * bvw, const char *name)
     }
   else
     {
-      g_message ("Failed loading visualization plugin %s", name);
       return FALSE;
     }
 }
@@ -1033,7 +1048,8 @@ bacon_video_widget_set_visuals_quality (BaconVideoWidget * bvw,
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
   g_return_if_fail (GST_IS_PLAY (bvw->priv->play));
 
-  g_message ("visual quality");
+  if (bvw->priv->vis_element == NULL)
+    return;
 
   switch (quality)
     {
@@ -1071,10 +1087,10 @@ bacon_video_widget_set_visuals_quality (BaconVideoWidget * bvw,
 gboolean
 bacon_video_widget_get_auto_resize (BaconVideoWidget * bvw)
 {
-  g_return_if_fail (bvw != NULL);
-  g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
-  g_return_if_fail (GST_IS_PLAY (bvw->priv->play));
-  g_return_if_fail (GST_IS_VIDEO_WIDGET (bvw->priv->vw));
+  g_return_val_if_fail (bvw != NULL, FALSE);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
+  g_return_val_if_fail (GST_IS_PLAY (bvw->priv->play), FALSE);
+  g_return_val_if_fail (GST_IS_VIDEO_WIDGET (bvw->priv->vw), FALSE);
 
   return gst_video_widget_get_auto_resize (bvw->priv->vw);
 }
@@ -1134,7 +1150,7 @@ bacon_video_widget_set_video_property (BaconVideoWidget *bvw,
 	//FIXME
 }
 
-int
+float
 bacon_video_widget_get_position (BaconVideoWidget * bvw)
 {
   g_return_val_if_fail (bvw != NULL, -1);
@@ -1142,7 +1158,7 @@ bacon_video_widget_get_position (BaconVideoWidget * bvw)
   return bvw->priv->current_position;
 }
 
-int
+gint64
 bacon_video_widget_get_current_time (BaconVideoWidget * bvw)
 {
   g_return_val_if_fail (bvw != NULL, -1);
@@ -1150,7 +1166,7 @@ bacon_video_widget_get_current_time (BaconVideoWidget * bvw)
   return bvw->priv->current_time;
 }
 
-int
+gint64
 bacon_video_widget_get_stream_length (BaconVideoWidget * bvw)
 {
   g_return_val_if_fail (bvw != NULL, -1);
@@ -1446,7 +1462,7 @@ bacon_video_widget_get_type (void)
       };
 
       bacon_video_widget_type = g_type_register_static
-	(GTK_TYPE_VBOX, "BaconVideoWidget",
+	(GTK_TYPE_BOX, "BaconVideoWidget",
 	 &bacon_video_widget_info, (GTypeFlags) 0);
     }
 
@@ -1458,7 +1474,7 @@ bacon_video_widget_new (int width, int height,
 			gboolean null_out, GError ** err)
 {
   BaconVideoWidget *bvw;
-  GstElement *audio_sink, *video_sink, *vis_video_sink, *vis_element;
+  GstElement *audio_sink, *video_sink, *vis_video_sink;
 
   bvw = BACON_VIDEO_WIDGET (g_object_new
 			    (bacon_video_widget_get_type (), NULL));
@@ -1515,6 +1531,9 @@ bacon_video_widget_new (int width, int height,
   g_signal_connect (G_OBJECT (bvw->priv->play),
 		    "have_video_size", (GtkSignalFunc) got_video_size,
 		    (gpointer) bvw);
+  g_signal_connect (G_OBJECT (bvw->priv->play),
+		    "have_vis_size", (GtkSignalFunc) got_vis_video_size,
+		    (gpointer) bvw);
   g_signal_connect (G_OBJECT (bvw->priv->play), "stream_end",
 		    (GtkSignalFunc) got_eos, (gpointer) bvw);
   g_signal_connect (G_OBJECT (bvw->priv->play), "stream_length",
