@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.79 2003/10/14 12:06:45 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.82 2003/11/02 01:58:05 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -581,6 +581,11 @@ load_video_out_driver (BaconVideoWidget *bvw, gboolean null_out)
 	 * we need to try loading the other ones, skipping dxr3 */
 
 	/* The types are hardcoded for now */
+	vo_driver = xine_open_video_driver (bvw->priv->xine, "xvmc",
+			XINE_VISUAL_TYPE_X11, (void *) &vis);
+	if (vo_driver)
+		return vo_driver;
+
 	vo_driver = xine_open_video_driver (bvw->priv->xine, "xv",
 			XINE_VISUAL_TYPE_X11, (void *) &vis);
 	if (vo_driver)
@@ -1223,6 +1228,7 @@ xine_event (void *user_data, const xine_event_t *event)
 		ref = event->data;
 		bacon_video_widget_close (bvw);
 		bacon_video_widget_open (bvw, ref->mrl, NULL);
+		bacon_video_widget_play (bvw, NULL);
 		break;
 	case XINE_EVENT_UI_MESSAGE:
 		xine_event_message (bvw, (xine_ui_message_data_t *)event->data);
@@ -1337,6 +1343,7 @@ GtkWidget *
 bacon_video_widget_new (int width, int height, gboolean null_out, GError **err)
 {
 	BaconVideoWidget *bvw;
+	xine_cfg_entry_t entry;
 
 	bvw = BACON_VIDEO_WIDGET (g_object_new
 			(bacon_video_widget_get_type (), NULL));
@@ -1372,6 +1379,18 @@ bacon_video_widget_new (int width, int height, gboolean null_out, GError **err)
 	//FIXME
 	g_assert (bvw->priv->vo_driver != NULL);
 
+	if (bvw->priv->null_out != FALSE)
+	{
+		bvw_config_help_num (bvw->priv->xine, "video.num_buffers",
+				5, &entry);
+		entry.num_value = 5;
+	} else {
+		bvw_config_help_num (bvw->priv->xine, "video.num_buffers",
+				500, &entry);
+		entry.num_value = 500;
+	}
+	xine_config_update_entry (bvw->priv->xine, &entry);
+
 	bvw->priv->stream = xine_stream_new (bvw->priv->xine,
 			bvw->priv->ao_driver, bvw->priv->vo_driver);
 	bvw->priv->ev_queue = xine_event_new_queue (bvw->priv->stream);
