@@ -46,6 +46,7 @@
 #define MIME_READ_CHUNK_SIZE 1024
 
 typedef gboolean (*PlaylistCallback) (TotemPlParser *parser, const char *url, gpointer data);
+static gboolean totem_pl_parser_ignore (const char *url);
 
 typedef struct {
 	char *mimetype;
@@ -244,7 +245,7 @@ totem_pl_parser_write (TotemPlParser *parser, GtkTreeModel *model,
 		return FALSE;
 	}
 
-	buf = g_strdup ("[parser]\n");
+	buf = g_strdup ("[playlist]\n");
 	success = write_string (handle, buf, error);
 	g_free (buf);
 	if (success == FALSE)
@@ -487,13 +488,18 @@ totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url, gpointer data)
 		if (strcmp (lines[i], "") == 0)
 			continue;
 
+		retval = TRUE;
+
 		/* Either it's a URI, or it has a proper path ... */
 		if (strstr(lines[i], "://") != NULL
 				|| lines[i][0] == G_DIR_SEPARATOR) {
 			/* We use the same code for .ram and m3u parsers,
 			 * and .ram files can contain .smil entries */
-			if (totem_pl_parser_parse (parser, lines[i]) == TRUE)
-				retval = TRUE;
+			if (totem_pl_parser_parse (parser, lines[i]) == FALSE)
+			{
+				totem_pl_parser_add_one_url (parser,
+						lines[i], NULL);
+			}
 		} else if (lines[i][0] == '\\' && lines[i][1] == '\\') {
 			/* ... Or it's in the windows smb form
 			 * (\\machine\share\filename), Note drive names
@@ -504,10 +510,13 @@ totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url, gpointer data)
 			lines[i] = g_strdelimit (lines[i], "\\", '/');
 			tmpurl = g_strjoin (NULL, "smb:", lines[i], NULL);
 
-			totem_pl_parser_add_one_url (parser, tmpurl, NULL);
+			totem_pl_parser_add_one_url (parser, lines[i], NULL);
 			retval = TRUE;
 
 			g_free (tmpurl);
+		} else if (strcmp (lines[i], "--stop--") == 0) {
+			/* For Real Media playlists, handle the stop command */
+			break;
 		} else {
 			/* Try with a base */
 			char *fullpath, *base, sep;
@@ -517,8 +526,11 @@ totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url, gpointer data)
 			if (sep == '\\')
 				lines[i] = g_strdelimit (lines[i], "\\", '/');
 			fullpath = g_strdup_printf ("%s/%s", base, lines[i]);
-			if (totem_pl_parser_parse (parser, fullpath) == TRUE)
-				retval = TRUE;
+			if (totem_pl_parser_parse (parser, fullpath) == FALSE)
+			{
+				totem_pl_parser_add_one_url (parser,
+						fullpath, NULL);
+			}
 			g_free (fullpath);
 			g_free (base);
 		}
@@ -591,8 +603,8 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 	g_free (contents);
 
 	/* [parser] */
-	if (g_ascii_strncasecmp (lines[0], "[parser]",
-				(gsize)strlen ("[parser]")) != 0)
+	if (g_ascii_strncasecmp (lines[0], "[playlist]",
+				(gsize)strlen ("[playlist]")) != 0)
 		goto bail;
 
 	/* numberofentries=? */
@@ -674,7 +686,8 @@ parse_asx_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 
 		fullpath = g_strdup_printf ("%s/%s", base, url);
 		/* .asx files can contain references to other .asx files */
-		totem_pl_parser_parse (parser, fullpath);
+		if (totem_pl_parser_parse (parser, fullpath) == FALSE)
+			totem_pl_parser_add_one_url (parser, fullpath, NULL);
 
 		g_free (fullpath);
 	}
@@ -917,8 +930,16 @@ totem_pl_parser_add_desktop (TotemPlParser *parser, const char *url, gpointer da
 		gnome_desktop_item_unref (ditem);
 		return FALSE;
 	}
+
+	retval = TRUE;
+
 	display_name = gnome_desktop_item_get_localestring (ditem, "Name");
-	retval = totem_pl_parser_add_url (parser, path, display_name);
+	if (totem_pl_parser_ignore (path) == FALSE) {
+		totem_pl_parser_add_one_url (parser, path, display_name);
+	} else {
+		if (totem_pl_parser_parse (parser, path) == FALSE)
+			totem_pl_parser_add_one_url (parser, path, display_name);
+	}
 	gnome_desktop_item_unref (ditem);
 
 	return retval;
